@@ -15,27 +15,23 @@ package remote
 
 import (
 	"context"
-	"io/ioutil"
 	"net/url"
-	"os"
-	"reflect"
 	"sort"
 	"testing"
 
 	"github.com/pkg/errors"
 	"github.com/prometheus/client_golang/prometheus"
 	config_util "github.com/prometheus/common/config"
+	"github.com/stretchr/testify/require"
+
 	"github.com/prometheus/prometheus/config"
-	"github.com/prometheus/prometheus/pkg/labels"
+	"github.com/prometheus/prometheus/model/labels"
 	"github.com/prometheus/prometheus/prompb"
 	"github.com/prometheus/prometheus/storage"
-	"github.com/prometheus/prometheus/util/testutil"
 )
 
 func TestNoDuplicateReadConfigs(t *testing.T) {
-	dir, err := ioutil.TempDir("", "TestNoDuplicateReadConfigs")
-	testutil.Ok(t, err)
-	defer os.RemoveAll(dir)
+	dir := t.TempDir()
 
 	cfg1 := config.RemoteReadConfig{
 		Name: "write-1",
@@ -95,7 +91,7 @@ func TestNoDuplicateReadConfigs(t *testing.T) {
 
 	for _, tc := range cases {
 		t.Run("", func(t *testing.T) {
-			s := NewStorage(nil, nil, nil, dir, defaultFlushDeadline)
+			s := NewStorage(nil, nil, nil, dir, defaultFlushDeadline, nil)
 			conf := &config.Config{
 				GlobalConfig:      config.DefaultGlobalConfig,
 				RemoteReadConfigs: tc.cfgs,
@@ -103,8 +99,8 @@ func TestNoDuplicateReadConfigs(t *testing.T) {
 			err := s.ApplyConfig(conf)
 			prometheus.Unregister(s.rws.highestTimestamp)
 			gotError := err != nil
-			testutil.Equals(t, tc.err, gotError)
-			testutil.Ok(t, s.Close())
+			require.Equal(t, tc.err, gotError)
+			require.NoError(t, s.Close())
 		})
 	}
 }
@@ -145,8 +141,8 @@ func TestExternalLabelsQuerierAddExternalLabels(t *testing.T) {
 		},
 		{
 			el: labels.Labels{
-				{Name: "region", Value: "europe"},
 				{Name: "dc", Value: "berlin-01"},
+				{Name: "region", Value: "europe"},
 			},
 			inMatchers: []*labels.Matcher{
 				labels.MustNewMatcher(labels.MatchEqual, "job", "api-server"),
@@ -170,12 +166,8 @@ func TestExternalLabelsQuerierAddExternalLabels(t *testing.T) {
 		sort.Slice(test.outMatchers, func(i, j int) bool { return test.outMatchers[i].Name < test.outMatchers[j].Name })
 		sort.Slice(matchers, func(i, j int) bool { return matchers[i].Name < matchers[j].Name })
 
-		if !reflect.DeepEqual(matchers, test.outMatchers) {
-			t.Fatalf("%d. unexpected matchers; want %v, got %v", i, test.outMatchers, matchers)
-		}
-		if !reflect.DeepEqual(added, test.added) {
-			t.Fatalf("%d. unexpected added labels; want %v, got %v", i, test.added, added)
-		}
+		require.Equal(t, test.outMatchers, matchers, "%d", i)
+		require.Equal(t, test.added, added, "%d", i)
 	}
 }
 
@@ -204,9 +196,9 @@ func TestSeriesSetFilter(t *testing.T) {
 	for _, tc := range tests {
 		filtered := newSeriesSetFilter(FromQueryResult(true, tc.in), tc.toRemove)
 		act, ws, err := ToQueryResult(filtered, 1e6)
-		testutil.Ok(t, err)
-		testutil.Equals(t, 0, len(ws))
-		testutil.Equals(t, tc.expected, act)
+		require.NoError(t, err)
+		require.Equal(t, 0, len(ws))
+		require.Equal(t, tc.expected, act)
 	}
 }
 
@@ -495,22 +487,21 @@ func TestSampleAndChunkQueryableClient(t *testing.T) {
 				tc.callback,
 			)
 			q, err := c.Querier(context.TODO(), tc.mint, tc.maxt)
-			testutil.Ok(t, err)
-			defer testutil.Ok(t, q.Close())
+			require.NoError(t, err)
+			defer require.NoError(t, q.Close())
 
 			ss := q.Select(true, nil, tc.matchers...)
-			testutil.Ok(t, err)
-			testutil.Equals(t, storage.Warnings(nil), ss.Warnings())
+			require.NoError(t, err)
+			require.Equal(t, storage.Warnings(nil), ss.Warnings())
 
-			testutil.Equals(t, tc.expectedQuery, m.got)
+			require.Equal(t, tc.expectedQuery, m.got)
 
 			var got []labels.Labels
 			for ss.Next() {
 				got = append(got, ss.At().Labels())
 			}
-			testutil.Ok(t, ss.Err())
-			testutil.Equals(t, tc.expectedSeries, got)
-
+			require.NoError(t, ss.Err())
+			require.Equal(t, tc.expectedSeries, got)
 		})
 	}
 }
